use std::hint::black_box;

use svg_filters::{
    codegen::SvgDocument,
    types::{graph::edge::Edge, nodes::primitives::color_matrix::ColorMatrixType},
    Node,
};

fn main() {
    // <filter id="chromabb" >
    //   <feColorMatrix values="1 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0" in="SourceGraphic" />
    //   <feOffset dx="25" dy="0" />
    //   <feGaussianBlur stdDeviation="5 0" result="red" />

    //   <feColorMatrix values="0 0 0 0 0 0 1 0 0 0 0 0 0 0 0 0 0 0 1 0" result="grn" in="SourceGraphic" />

    //   <feColorMatrix values="0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 0 0 1 0" in="SourceGraphic" />
    //   <feOffset dx="-25" dy="0" />
    //   <feGaussianBlur stdDeviation="5 0" result="blu"/>

    //   <feComposite in="red" in2="blu" operator="arithmetic" k2="1" k3="1" result="rb" />
    //   <feComposite in="rb" in2="grn" operator="arithmetic" k2="1" k3="1" />
    // </filter>

    let mut doc = SvgDocument::new();
    let chromabb = doc.create_filter("chromabb_gen");

    let chan_r = chromabb.add_node(Node::color_matrix(ColorMatrixType::Matrix(Box::new([
        1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1., 0.,
    ]))));
    let chan_g = chromabb.add_node(Node::color_matrix(ColorMatrixType::Matrix(Box::new([
        0., 0., 0., 0., 0., 0., 1., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1., 0.,
    ]))));
    let chan_b = chromabb.add_node(Node::color_matrix(ColorMatrixType::Matrix(Box::new([
        0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1., 0., 0., 0., 0., 0., 1., 0.,
    ]))));

    let offset_r = chromabb.add_node(Node::offset(25., 0.));
    let offset_b = chromabb.add_node(Node::offset(-25., 0.));
    let blur_r = chromabb.add_node(Node::gaussian_blur_xy(5, 0));
    let blur_b = chromabb.add_node(Node::gaussian_blur_xy(5, 0));

    let composite_rb = chromabb.add_node(Node::composite_arithmetic(0., 1., 1., 0.));
    let composite_final = chromabb.add_node(Node::composite_arithmetic(0., 1., 1., 0.));

    chromabb.dag.extend_with_edges(&[
        (chromabb.source_graphic(), chan_r),
        (chromabb.source_graphic(), chan_b),
        (chromabb.source_graphic(), chan_g),
        (chan_r, offset_r),
        (offset_r, blur_r),
        (chan_b, offset_b),
        (offset_b, blur_b),
    ]);
    chromabb.dag.extend_with_edges(&[
        (blur_r, composite_rb, Edge::new(0)),
        (blur_b, composite_rb, Edge::new(1)),
        (composite_rb, composite_final, Edge::new(0)),
        (chan_g, composite_final, Edge::new(1)),
    ]);

    black_box(doc.generate_svg());
}
