pub mod length;
pub mod nodes;

// pub mod old;

pub mod graph {
    use std::iter::Iterator;

    use petgraph::{data::Build, prelude::*};

    use crate::Node;

    use self::edge::Edge;

    use super::nodes::standard_input::StandardInput;

    #[derive(Debug)]
    pub struct FilterGraph {
        pub dag: DiGraph<Node, Edge>,
        source_graphic_idx: NodeIndex,
    }

    impl FilterGraph {
        pub fn new() -> Self {
            let mut dag = DiGraph::new();
            let source_graphic_idx = dag.add_node(Node::StdInput(StandardInput::SourceGraphic));

            Self {
                dag,
                source_graphic_idx,
            }
        }

        pub fn add_node(&mut self, node: Node) -> NodeIndex {
            self.dag.add_node(node)
        }

        pub fn source_graphic(&self) -> NodeIndex {
            self.source_graphic_idx
        }
    }

    pub mod edge {
        #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
        pub struct Edge {
            input_idx: u8,
        }

        impl Edge {
            pub fn new(input_idx: u8) -> Self {
                Self { input_idx }
            }
        }

        impl Default for Edge {
            fn default() -> Self {
                Self { input_idx: 0 }
            }
        }

        impl ToString for Edge {
            fn to_string(&self) -> String {
                match self.input_idx {
                    0 => "in".to_owned(),
                    n => format!("in{}", n + 1),
                }
            }
        }
    }
}
